c     Copyright (C)
c     2008-2013 Universit Paul Sabatier
c     2008-2013 Centre National de la Recherche Scientifique
c     2008-2013 Ecole des Mines d'Albi
c     2008-2013 Universit de Bordeaux 1
c     2008-2013 Fondation STAE
c     2014-2018 Meso-Star
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c     
      subroutine Lorentz_linewidth(Tref,T,P,Ps,n_air,n_self,
     &     gamma_air,gamma_self,gamma_L)
      implicit none
c     
c     Purpose: to compute the Lorentz line width
c
c     Inputs:
c       + Tref: reference temperature [K]
c       + T: temperature [K]
c       + P: pressure [atm]
c       + Ps: partial pressure of the given molecular species [atm]
c       + n_air: temperature-dependance exponent for gamma_air [unitless]
c       + n_self: temperature-dependance exponent for gamma_self [unitless]
c       + gamma_air: air-broadened half-width [cm.atm]
c       + gamma_self: self-broadened half-width [cm.atm]
c
c     Outputs:
c       + gamma_L: Lorentz half-width [cm]
c
      include 'parameters.inc'
      double precision Tref
      double precision T
      double precision P
      double precision Ps
      double precision n_air
      double precision n_self
      double precision gamma_air
      double precision gamma_self
      double precision gamma_L

      gamma_L=gamma_air*(P-Ps)*(Tref/T)**n_air
     &     +gamma_self*Ps*(Tref/T)**n_self      ! half-width [cm]

      return
      end
