c     Copyright (C)
c     2008-2013 Universit Paul Sabatier
c     2008-2013 Centre National de la Recherche Scientifique
c     2008-2013 Ecole des Mines d'Albi
c     2008-2013 Universit de Bordeaux 1
c     2008-2013 Fondation STAE
c     2014-2018 Meso-Star
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c     
      subroutine read_truncation_distances(Hnmol,Hmol_name,Hmol_index,
     &     default_tr_dist,tr_dist)
      implicit none
      include 'max.inc'
c     
c     Purpose: to read the data/truncation_distances.in file if present
c     
c     Input:
c       + Hnmol: number of HITRAN molecules
c       + Hmol_name: name of the HITRAN molecules
c       + Hmol_index: indexes of the HITRAN molecules
c       + default_tr_dist: default truncation distance [inv. cm]
c     
c     Output:
c       + tr_dist: truncation distance [inv. cm] for each molecular species
c     
c     I/O
      integer Hnmol
      character*10 Hmol_name(1:Nmol_max)
      integer Hmol_index(1:Nmol_max)
      double precision default_tr_dist
      double precision tr_dist(1:Nmol_max)
c     temp
      integer i,nl,Nm,hm,mol_index
      logical mf
      character*(Nchar_mx) tr_file
      logical trf_ex
      character*(Nchar_mx) string,stringp_file
      integer nmolec
      character*10 molec_name(1:Nmol_max)
      character*10 cmol,hmol
c     functions
      logical is_even
c     label
      character*(Nchar_mx) label
      label='subroutine read_truncation_distances'

      tr_file='./data/truncation_distances.in'
      inquire(file=trim(tr_file),exist=trf_ex)
c     read file only if it exists
      if (trf_ex) then
         stringp_file='./nlines'
         call get_nlines(tr_file,stringp_file,nl)
c     This file should have a first comment line, then two lines of data for a
c     number of molecular species: its number of lines should therefore be
c     at least 3, and not be a even number.
         if (nl.lt.3) then
            call error(label)
            write(*,*) 'Truncation distance file:'
            write(*,*) trim(tr_file)
            write(*,*) 'has been found, but does not seem to '
     &           //'hold enough information'
            stop
         endif
         if (is_even(nl)) then
            call error(label)
            write(*,*) 'Truncation distance file:'
            write(*,*) trim(tr_file)
            write(*,*) 'has been found, but does not seem to '
     &           //'comply with requirements'
            stop
         endif
c     Defalut values
         do i=1,Nmol_max
            tr_dist(i)=default_tr_dist
         enddo                  ! i
c     Reading values per molecule
         Nm=(nl-1)/2
         open(11,file=trim(tr_file))
         read(11,*)
         do i=1,Nm
            read(11,'(a)') string
c     Identify the molecule from the provided character string
            mf=.false.
            cmol=trim(string)
            do hm=1,Hnmol
               hmol=Hmol_name(hm)
               if (trim(cmol).eq.trim(hmol)) then
                  mf=.true.
                  mol_index=Hmol_index(hm)
                  goto 111
               endif
            enddo               ! hm         
 111        continue
            if (.not.mf) then
               call error(label)
               write(*,*) 'molecule: "',trim(cmol),
     &              '" could not be identified'
               write(*,*) 'within the list of HITRAN molecules'
               stop
            endif
c     read the truncation distance for molecule index "mol_index"
            read(11,*) tr_dist(mol_index)
c     
         enddo ! i
         close(11)
      endif
      
      return
      end
      
